﻿using STS.General.Generators;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Xml.Serialization;
using VelocityDb;
using VelocityDb.Collection.BTree;
using VelocityDb.Session;

namespace DatabaseBenchmark.Databases
{
    public class VelocityDBDatabase : Database
    {
        protected UInt64 m_mapId;
        private SessionBase m_session;
        object m_lockObj;

        public override IndexingTechnology IndexingTechnology
        {
            get { return IndexingTechnology.BTree; }
        }

        public VelocityDBDatabase()
        {
            Name = "VelocityDB";
            Category = "NoSQL\\Object Databases";
            Description = "VelocityDB Standalone Client 5.3.12 April 26, 2016";
            Website = "http://VelocityDB.com/";
            Color = Color.FromArgb(141, 186, 137);

            Requirements = new string[]
            { 
                "VelocityDB.dll"
            };
        }

        public override void Init(int flowCount, long flowRecordCount)
        {
            m_mapId = new UInt64();
            SessionBase.DefaultCompressPages = PageInfo.compressionKind.LZ4;
            m_session = new SessionNoServer(DataDirectory);
            m_session.BeginUpdate();
            m_session.RegisterClass(typeof(VelocityTick));
            m_session.RegisterClass(typeof(BTreeMapOidShort<long, VelocityTick>));
            m_session.Commit();
            m_session.BeginUpdate();
        }

        public override void Write(int flowID, IEnumerable<KeyValuePair<long, Tick>> flow)
        {
            Placement place = new Placement((ushort)checked(flowID + 10));
            Placement velocityRecordPlace = new Placement((ushort)(flowID + 10), 1000, 1, 10000, ushort.MaxValue, false, false, UInt32.MaxValue, false);

            BTreeMapOidShort<long, VelocityTick> map = new BTreeMapOidShort<long, VelocityTick>(null, m_session, 1000);

            m_mapId = map.Persist(place, m_session);
            map.ValuePlacement = velocityRecordPlace;
            map.TransientBatchSize = 2000;

            foreach (var kv in flow)
            {
                VelocityTick tick = new VelocityTick(kv.Value);
                map.AddFast(kv.Key, tick);
            }
        }

        public override IEnumerable<KeyValuePair<long, Tick>> Read()
        {
            BTreeMapOidShort<long, VelocityTick> map = m_session.Open<BTreeMapOidShort<long, VelocityTick>>(m_mapId);

            foreach (KeyValuePair<long, VelocityTick> pair in map)
            {
                yield return new KeyValuePair<long, Tick>(pair.Key, pair.Value.Record);
            }
        }

        public override void Finish()
        {
            if (m_session.InTransaction)
                m_session.Commit();
        }

        [XmlIgnore]
        public override Dictionary<string, string> Settings
        {
            get
            {
                return null;
            }
        }

        public class VelocityTick : OptimizedPersistable
        {
            string m_symbol;
            DateTime m_timestamp;
            double m_bid;
            double m_ask;
            int m_bidSize;
            int m_askSize;
            string m_provider;

            public VelocityTick(Tick tick)
            {
                m_symbol = tick.Symbol;
                m_timestamp = tick.Timestamp;
                m_bid = tick.Bid;
                m_ask = tick.Ask;
                m_askSize = tick.AskSize;
                m_bidSize = tick.BidSize;
                m_provider = tick.Provider;
            }

            public Tick Record
            {
                get { return new Tick(m_symbol, m_timestamp, m_bid, m_ask, m_bidSize, m_askSize, m_provider); }
            }

            public override bool AllowOtherTypesOnSamePage
            {
                get { return false; }
            }
        }
    }
}